/* place.c generated by valac 0.28.0, the Vala compiler
 * generated from place.vala, do not modify */

/*
 * Seahorse
 *
 * Copyright (C) 2004,2005 Stefan Walter
 * Copyright (C) 2011 Collabora Ltd.
 * Copyright (C) 2013 Stef Walter
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */

#include <glib.h>
#include <glib-object.h>
#include "seahorse-common.h"
#include <gio/gio.h>






void seahorse_place_load (SeahorsePlace* self, GCancellable* cancellable, GAsyncReadyCallback _callback_, gpointer _user_data_) {
	SEAHORSE_PLACE_GET_INTERFACE (self)->load (self, cancellable, _callback_, _user_data_);
}


gboolean seahorse_place_load_finish (SeahorsePlace* self, GAsyncResult* _res_, GError** error) {
	return SEAHORSE_PLACE_GET_INTERFACE (self)->load_finish (self, _res_, error);
}


gchar* seahorse_place_get_label (SeahorsePlace* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return SEAHORSE_PLACE_GET_INTERFACE (self)->get_label (self);
}


gchar* seahorse_place_get_description (SeahorsePlace* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return SEAHORSE_PLACE_GET_INTERFACE (self)->get_description (self);
}


gchar* seahorse_place_get_uri (SeahorsePlace* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return SEAHORSE_PLACE_GET_INTERFACE (self)->get_uri (self);
}


GIcon* seahorse_place_get_icon (SeahorsePlace* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return SEAHORSE_PLACE_GET_INTERFACE (self)->get_icon (self);
}


GtkActionGroup* seahorse_place_get_actions (SeahorsePlace* self) {
	g_return_val_if_fail (self != NULL, NULL);
	return SEAHORSE_PLACE_GET_INTERFACE (self)->get_actions (self);
}


static void seahorse_place_base_init (SeahorsePlaceIface * iface) {
	static gboolean initialized = FALSE;
	if (!initialized) {
		initialized = TRUE;
		g_object_interface_install_property (iface, g_param_spec_string ("label", "label", "label", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
		g_object_interface_install_property (iface, g_param_spec_string ("description", "description", "description", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
		g_object_interface_install_property (iface, g_param_spec_string ("uri", "uri", "uri", NULL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
		g_object_interface_install_property (iface, g_param_spec_object ("icon", "icon", "icon", G_TYPE_ICON, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
		g_object_interface_install_property (iface, g_param_spec_object ("actions", "actions", "actions", gtk_action_group_get_type (), G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	}
}


GType seahorse_place_get_type (void) {
	static volatile gsize seahorse_place_type_id__volatile = 0;
	if (g_once_init_enter (&seahorse_place_type_id__volatile)) {
		static const GTypeInfo g_define_type_info = { sizeof (SeahorsePlaceIface), (GBaseInitFunc) seahorse_place_base_init, (GBaseFinalizeFunc) NULL, (GClassInitFunc) NULL, (GClassFinalizeFunc) NULL, NULL, 0, 0, (GInstanceInitFunc) NULL, NULL };
		GType seahorse_place_type_id;
		seahorse_place_type_id = g_type_register_static (G_TYPE_INTERFACE, "SeahorsePlace", &g_define_type_info, 0);
		g_type_interface_add_prerequisite (seahorse_place_type_id, gcr_collection_get_type ());
		g_once_init_leave (&seahorse_place_type_id__volatile, seahorse_place_type_id);
	}
	return seahorse_place_type_id__volatile;
}



