/* icons.c generated by valac 0.28.0, the Vala compiler
 * generated from icons.vala, do not modify */

/*
 * Seahorse
 *
 * Copyright (C) 2011 Collabora Ltd.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, see <http://www.gnu.org/licenses/>.
 *
 * Author: Stef Walter <stefw@collabora.co.uk>
 */

#include <glib.h>
#include <glib-object.h>
#include "seahorse-common.h"
#include <stdlib.h>
#include <string.h>
#include "config.h"
#include <gtk/gtk.h>

#define _g_free0(var) (var = (g_free (var), NULL))


gboolean seahorse__icons_inited = FALSE;



void seahorse_icons_init (void) {
	gboolean _tmp0_ = FALSE;
	gchar* path = NULL;
	gchar* _tmp1_ = NULL;
	GtkIconTheme* _tmp2_ = NULL;
	const gchar* _tmp3_ = NULL;
	_tmp0_ = seahorse__icons_inited;
	if (_tmp0_) {
		return;
	}
	seahorse__icons_inited = TRUE;
	_tmp1_ = g_build_filename (PKGDATADIR, "icons", NULL);
	path = _tmp1_;
	_tmp2_ = gtk_icon_theme_get_default ();
	_tmp3_ = path;
	gtk_icon_theme_append_search_path (_tmp2_, _tmp3_);
	gtk_window_set_default_icon_name ("seahorse");
	_g_free0 (path);
}



